@@ -42,15 +42,17 @@ class RoWordNet(object):
             path = "rowordnet.pickle"  # always use slash
             filepath = pkg_resources.resource_filename(__name__, path)
             self._load_from_binary(filepath)
-            return
+        else:
+            if xml is True:
+                self._load_from_xml(filename)
+            else:
+                self._load_from_binary(filename)
 
-        if xml is True:
-            self._load_from_xml(filename)
-        elif xml is False:
-            self._load_from_binary(filename)
+        self._max_hypernym_height = self._hypernym_tree_height("ENG30-00002684-n")
 
     def _clean(self):
         self._graph = nx.DiGraph()
+        self._hypernym_graph = nx.DiGraph()
         self._synsets = {}
         self._literal2synset = defaultdict(list)
         self._literal2synset_strict = defaultdict(list)
@@ -197,10 +199,16 @@ class RoWordNet(object):
         synsets_id = wn.synsets()
 
         for synset_id in synsets_id:
+            self._graph.add_node(synset_id)
+            self._hypernym_graph.add_node(synset_id)
+
             adj_synsets_id = wn.outbound_relations(synset_id)
             for adj_synset_id, relation in adj_synsets_id:
                 self._graph.add_edge(synset_id, adj_synset_id, label=relation)
 
+                if relation == "hypernym" or relation == "hyponym":
+                    self._hypernym_graph.add_edge(synset_id, adj_synset_id, label=relation)
+
         for synset_id in synsets_id:
             self._synsets[synset_id] = wn.synset(synset_id)
 
@@ -315,7 +323,7 @@ class RoWordNet(object):
 
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given id in the wordnet.
+                WordNetError: If there's no synset with the given id in the wordnet.
         """
 
         if not isinstance(synset_id, str):
@@ -554,7 +562,7 @@ class RoWordNet(object):
                 relation(str): Relation type between the synsets.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given ids in the wordnet, if there's already a relation from
+                WordNetError: If there's no synset with the given ids in the wordnet, if there's already a relation from
                     the first synset to the second synset or if the given relation has an incorrect value.
         """
 
@@ -587,7 +595,7 @@ class RoWordNet(object):
                 synset_id2 (str): Id of the second synset.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given ids in the wordnet or if there's no relation from the
+                WordNetError: If there's no synset with the given ids in the wordnet or if there's no relation from the
                     first synset to the second synset.
         """
 
@@ -616,7 +624,7 @@ class RoWordNet(object):
                 list: A list containing synset ids that create the path to the root of the tree.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given id in the wordnet.
+                WordNetError: If there's no synset with the given id in the wordnet.
         """
 
         if not isinstance(synset_id, str):
@@ -629,11 +637,10 @@ class RoWordNet(object):
         synset_id_to_root = [synset_id]
 
         while synset_id_ancestor is not None:
-            adj_synsets_id = self._graph[synset_id_ancestor]
-
+            adj_synsets = self._hypernym_graph[synset_id_ancestor]
             synset_id_ancestor = None
 
-            for adj_synset_id, adj_synset_data in adj_synsets_id.items():
+            for adj_synset_id, adj_synset_data in adj_synsets.items():
                 if adj_synset_data['label'] == 'hypernym':
                     synset_id_to_root.append(adj_synset_id)
                     synset_id_ancestor = adj_synset_id
@@ -651,7 +658,7 @@ class RoWordNet(object):
                 str: A synset ID representing the lowest common ancestor in the specified tree.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given ids in the wordnet.value.
+                WordNetError: If there's no synset with the given ids in the wordnet.value.
         """
 
         synset_id1_to_root = self.synset_to_hypernym_root(synset_id1)
@@ -672,7 +679,7 @@ class RoWordNet(object):
                 Synset: The next synset in the wordnet.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given id in the wordnet.
+                WordNetError: If there's no synset with the given id in the wordnet.
         """
 
         if not isinstance(synset_id, str):
@@ -717,8 +724,9 @@ class RoWordNet(object):
                 the first synset to the second synset.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
+                WordNetError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
                     value.
+                networkx.exception.NetworkXNoPath: If there is no path between the synsets.
         """
 
         if not isinstance(synset_id1, str):
@@ -746,46 +754,24 @@ class RoWordNet(object):
                 if relation not in self._relation_types:
                     raise WordNetError("Relation '{}' is not a correct relation".format(relation))
 
-            queue = Queue()
-            queue.put(synset_id1)
-            from_synset_id = {}
-            marked_synset_ids = [synset_id1]
-            found = False
+            if relations == {"hypernym", "hyponym"}:
+                return nx.shortest_path(self._hypernym_graph, synset_id1, synset_id2)
+            else:
+                raise NotImplemented("The current set of relations is not supported anymore by the function.")
 
-            while not queue.empty() and not found:
-                cur_synset_id = queue.get()
-
-                for adj_synset_id, data in self._graph.adj[cur_synset_id].items():
-                    if data['label'] in relations and adj_synset_id not in marked_synset_ids:
-                        from_synset_id[adj_synset_id] = cur_synset_id
-                        queue.put(adj_synset_id)
-                        marked_synset_ids.append(adj_synset_id)
-
-                        if adj_synset_id == synset_id2:
-                            found = True
-
-            shortest_path_list = [synset_id2]
-            cur_synset_id = synset_id2
-
-            while not cur_synset_id == synset_id1:
-                cur_synset_id = from_synset_id[cur_synset_id]
-                shortest_path_list.append(cur_synset_id)
-
-            shortest_path_list.reverse()
-            return shortest_path_list
-
-    def path_similarity(self, synset_id1: str, synset_id2: str):
+    def path_similarity(self, synset_id1: str, synset_id2: str, simulate_root: bool = True):
         """
             Returns the path similarity between two synsets.
             Args:
                 synset_id1 (str): Id of the first synset.
                 synset_id2 (str): Id of the second synset.
+                simulate_root (bool): Simulate a virtual root if there is no common root for the two synsets.
             Returns:
                 float: 1 if the synsets are equal, None if there is no path between the synsets or
                        1 / 1/(shortest_path_distance + 1) otherwise.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
+                WordNetError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
                     value.
         """
         if not isinstance(synset_id1, str):
@@ -806,22 +792,29 @@ class RoWordNet(object):
         try:
             shortest_path_distance = len(self.shortest_path(synset_id1, synset_id2, relations={"hypernym", "hyponym"}))
         except nx.exception.NetworkXNoPath:
-            return None
+            if simulate_root:
+                depth_synset1 = len(self.synset_to_hypernym_root(synset_id1))
+                depth_synset2 = len(self.synset_to_hypernym_root(synset_id2))
+
+                shortest_path_distance = depth_synset1 + depth_synset2 + 2
+            else:
+                return None
 
         return 1 / (shortest_path_distance + 1)
 
-    def wup_similarity(self, synset_id1: str, synset_id2: str):
+    def wup_similarity(self, synset_id1: str, synset_id2: str, simulate_root: bool = True):
         """
            Returns the Wu and Palmer similarity between two synsets.
            Args:
                synset_id1 (str): Id of the first synset.
                synset_id2 (str): Id of the second synset.
+               simulate_root (bool): Simulate a virtual root if there is no common root for the two synsets.
            Returns:
                float: None if there is no path between the synsets or distance between synsets defined as
                       2 * depth(least_common_subsumer(synset1,synset2))) / (depth(synset1) + depth(synset2)) otherwise.
            Raises:
                TypeError: If any argument has incorrect type.
-               WordNerError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
+               WordNetError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
                    value.
         """
         if not isinstance(synset_id1, str):
@@ -836,11 +829,23 @@ class RoWordNet(object):
         if synset_id2 not in self._synsets:
             raise WordNetError("Synset with id '{}' is not in the wordnet".format(synset_id2))
 
+        if synset_id1[-1] != synset_id2[-1]:
+            return None
+
+        lcs_synset = self.lowest_hypernym_common_ancestor(synset_id1, synset_id2)
+
         depth_synset1 = len(self.synset_to_hypernym_root(synset_id1))
         depth_synset2 = len(self.synset_to_hypernym_root(synset_id2))
 
-        lcs_synset = self.lowest_hypernym_common_ancestor(synset_id1, synset_id2)
-        depth_lcs_synset = len(self.synset_to_hypernym_root(lcs_synset))
+        if lcs_synset is None:
+            if simulate_root:
+                depth_synset1 += 1
+                depth_synset2 += 1
+                depth_lcs_synset = 1
+            else:
+                return None
+        else:
+            depth_lcs_synset = len(self.synset_to_hypernym_root(lcs_synset))
 
         return 2 * depth_lcs_synset / (depth_synset1 + depth_synset2)
 
@@ -856,18 +861,19 @@ class RoWordNet(object):
 
         return max(depths) + 1
 
-    def lch_similarity(self, synset_id1: str, synset_id2: str):
+    def lch_similarity(self, synset_id1: str, synset_id2: str, simulate_root: bool = True):
         """
             Returns the Leacock and Chodorow similarity between two synsets.
             Args:
                 synset_id1 (str): Id of the first synset.
                 synset_id2 (str): Id of the second synset.
+                simulate_root (bool): Simulate a virtual root if there is no common root for the two synsets.
             Returns:
                 float: None if there is no path between the synsets or distance between synsets defined as
                        -log ((dist(synset1,synset2)+1) / (2 * maximum taxonomy depth)) otherwise.
             Raises:
                 TypeError: If any argument has incorrect type.
-                WordNerError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
+                WordNetError: If there's no synset with the given ids in the wordnet or if any relation has an incorrect
                     value.
         """
         if not isinstance(synset_id1, str):
@@ -882,15 +888,22 @@ class RoWordNet(object):
         if synset_id2 not in self._synsets:
             raise WordNetError("Synset with id '{}' is not in the wordnet".format(synset_id2))
 
+        max_hypernym_height = self._max_hypernym_height
+
         try:
             shortest_path_distance = len(self.shortest_path(synset_id1, synset_id2, relations={"hypernym", "hyponym"}))
         except nx.exception.NetworkXNoPath:
-            return None
+            if simulate_root:
+                depth_synset1 = len(self.synset_to_hypernym_root(synset_id1))
+                depth_synset2 = len(self.synset_to_hypernym_root(synset_id2))
 
-        root_id = "ENG30-00002684-n"
-        max_hypernym_depth = self._hypernym_tree_height(root_id)
+                shortest_path_distance = depth_synset1 + depth_synset2 + 2
 
-        return - math.log2((shortest_path_distance + 1) / (2 * max_hypernym_depth))
+                max_hypernym_height += 1
+            else:
+                return None
+
+        return - math.log2((shortest_path_distance + 1) / (2 * max_hypernym_height))
 
 
 def intersection(wordnet_1, wordnet_2):
